@@ -17,6 +17,38 @@ delimiter ||;
 
 use mtr||
 
+CREATE DEFINER=root@localhost PROCEDURE check_testcase_perfschema()
+BEGIN
+  IF ((SELECT count(*) from information_schema.engines
+       where engine='PERFORMANCE_SCHEMA' and support='YES') = 1) THEN
+  BEGIN
+
+    BEGIN
+      -- For tests tampering with performance_schema table structure
+      DECLARE CONTINUE HANDLER for SQLEXCEPTION
+      BEGIN
+      END;
+
+      -- Leave the instruments in the same state
+      SELECT * from performance_schema.setup_instruments
+        where enabled='NO' order by NAME;
+    END;
+
+    -- Leave the consumers in the same state
+    SELECT * from performance_schema.setup_consumers
+      order by NAME;
+
+    -- Leave the actors setup in the same state
+    SELECT * from performance_schema.setup_actors
+      order by USER, HOST;
+
+    -- Leave the objects setup in the same state
+    SELECT * from performance_schema.setup_objects
+      order by OBJECT_TYPE, OBJECT_SCHEMA, OBJECT_NAME;
+  END;
+  END IF;
+END||
+
 --
 -- Procedure used to check if server has been properly
 -- restored after testcase has been run
@@ -24,11 +56,15 @@ use mtr||
 CREATE DEFINER=root@localhost PROCEDURE check_testcase()
 BEGIN
 
-  -- Dump all global variables except those
-  -- that are supposed to change
+  CALL check_testcase_perfschema();
+
+  -- Dump all global variables except those that may change.
+  -- timestamp changes if time passes. server_uuid changes if server restarts.
   SELECT * FROM INFORMATION_SCHEMA.GLOBAL_VARIABLES
-    WHERE variable_name NOT IN ('timestamp', 'innodb_file_format_max')
-      ORDER BY VARIABLE_NAME;
+    WHERE variable_name NOT IN ('timestamp', 'server_uuid',
+                                'innodb_file_format_max',
+                                'gtid_executed', 'gtid_purged')
+    ORDER BY VARIABLE_NAME;
 
   SELECT * FROM INFORMATION_SCHEMA.SESSION_VARIABLES
     WHERE variable_name = 'debug_sync';
@@ -47,7 +83,7 @@ BEGIN
       WHERE table_schema='mysql' AND table_name != 'ndb_apply_status'
         ORDER BY tables_in_mysql;
   SELECT CONCAT(table_schema, '.', table_name) AS columns_in_mysql,
-  	 column_name, ordinal_position, column_default, is_nullable,
+         column_name, ordinal_position, column_default, is_nullable,
          data_type, character_maximum_length, character_octet_length,
          numeric_precision, numeric_scale, character_set_name,
          collation_name, column_type, column_key, extra, column_comment
@@ -57,12 +93,18 @@ BEGIN
 
   -- Dump all events, there should be none
   SELECT * FROM INFORMATION_SCHEMA.EVENTS;
-  -- Dump all triggers	 except mtr internals, there should be none
+  -- Dump all triggers except mtr internals, there should be none
   SELECT * FROM INFORMATION_SCHEMA.TRIGGERS
          WHERE TRIGGER_NAME NOT IN ('gs_insert', 'ts_insert');
   -- Dump all created procedures, there should be none
   SELECT * FROM INFORMATION_SCHEMA.ROUTINES;
 
+  -- Dump all created compression dictionaries if InnoDB is enabled
+  IF ((SELECT COUNT(*) FROM information_schema.engines
+       WHERE engine = 'InnoDB' AND support IN ('YES', 'DEFAULT')) = 1) THEN
+    SELECT * FROM information_schema.xtradb_zip_dict ORDER BY name;
+  END IF;
+
   SHOW STATUS LIKE 'slave_open_temp_tables';
 
   -- Checksum system tables to make sure they have been properly
