@@ -32,7 +32,7 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
     along with Percona TokuBackup.  If not, see <http://www.gnu.org/licenses/>.
 ======= */
 
-#ident "$Id: 04ed43283ccfd8fc9fbe6ed0f0c63e749956d112 $"
+#ident "$Id: cae3a0878a13d4952fc09103bdb6343efb092924 $"
 
 #include "backup_debug.h"
 #include "file_hash_table.h"
@@ -212,6 +212,7 @@ disable_out: // preserves r if r!=0
         while (m_keep_capturing) sched_yield();
             }) );
 
+    calls->before_stop_capt_call();
     {
         with_rwlock_wrlocked ms(&m_session_rwlock, BACKTRACE(NULL));
 
@@ -226,6 +227,7 @@ disable_out: // preserves r if r!=0
         delete m_session;
         m_session = NULL;
     }
+    calls->after_stop_capt_call();
 
 unlock_out: // preserves r if r!0
 
