@@ -393,7 +393,7 @@ fi
 # Configure mysqld command line
 mysqld_bootstrap="${MYSQLD_BOOTSTRAP-$mysqld}"
 mysqld_install_cmd_line="$mysqld_bootstrap $defaults $mysqld_opt --bootstrap \
-  --basedir=$basedir --datadir=$ldata --log-warnings=0 --loose-skip-innodb \
+  --basedir=$basedir --datadir=$ldata --log-warnings=0 \
   --loose-skip-ndbcluster $args --max_allowed_packet=8M \
   --default-storage-engine=myisam \
   --net_buffer_length=16K"
@@ -410,7 +410,7 @@ else
   echo
   echo "You can try to start the mysqld daemon with:"
   echo
-  echo "    shell> $mysqld --skip-grant &"
+  echo "    shell> $mysqld --skip-grant-tables &"
   echo
   echo "and use the command line tool $bindir/mysql"
   echo "to connect to the mysql database and look at the grant tables:"
@@ -427,7 +427,7 @@ else
   echo "MySQL email archives available at http://lists.mysql.com/."
   echo
   echo "Please check all of the above before submitting a bug report at"
-  echo "  https://jira.percona.com/projects/PS"
+  echo "  https://bugs.launchpad.net/percona-server/+filebug"
   echo
   exit 1
 fi
@@ -479,7 +479,7 @@ then
 
   echo
   echo "Please report any problems at"
-  echo "  https://jira.percona.com/projects/PS"
+  echo "  https://bugs.launchpad.net/percona-server/+filebug"
   echo
   echo "Percona recommends that all production deployments be protected with a support"
   echo "contract (http://www.percona.com/mysql-suppport/) to ensure the highest uptime,"
