@@ -18,6 +18,7 @@
 
 #include "sys_vars.h"
 #include "my_stacktrace.h"
+#include "global_threads.h"
 
 #ifdef __WIN__
 #include <crtdbg.h>
@@ -34,9 +35,6 @@
 static volatile sig_atomic_t segfaulted= 0;
 extern ulong max_used_connections;
 extern volatile sig_atomic_t calling_initgroups;
-#ifdef HAVE_NPTL
-extern volatile sig_atomic_t ld_assume_kernel_is_set;
-#endif
 
 /**
  * Handler for fatal signals
@@ -116,7 +114,7 @@ extern "C" sig_handler handle_fatal_signal(int sig)
                         (uint) thread_scheduler->max_threads +
                         (uint) extra_max_connections);
 
-  my_safe_printf_stderr("thread_count=%u\n", (uint) thread_count);
+  my_safe_printf_stderr("thread_count=%u\n", get_thread_count());
 
   my_safe_printf_stderr("connection_count=%u\n", (uint) connection_count);
 
@@ -180,6 +178,9 @@ extern "C" sig_handler handle_fatal_signal(int sig)
     case THD::KILL_QUERY:
       kreason= "KILL_QUERY";
       break;
+    case THD::KILL_TIMEOUT:
+      kreason= "KILL_TIMEOUT";
+      break;
     case THD::KILLED_NO_VALUE:
       kreason= "KILLED_NO_VALUE";
       break;
@@ -189,7 +190,7 @@ extern "C" sig_handler handle_fatal_signal(int sig)
       "Some pointers may be invalid and cause the dump to abort.\n");
 
     my_safe_printf_stderr("Query (%p): ", thd->query());
-    my_safe_print_str(thd->query(), min(1024U, thd->query_length()));
+    my_safe_print_str(thd->query(), MY_MIN(1024U, thd->query_length()));
     my_safe_printf_stderr("Connection ID (thread ID): %lu\n",
                           (ulong) thd->thread_id);
     my_safe_printf_stderr("Status: %s\n\n", kreason);
@@ -215,21 +216,6 @@ extern "C" sig_handler handle_fatal_signal(int sig)
   }
 #endif
 
-#ifdef HAVE_NPTL
-  if (thd_lib_detected == THD_LIB_LT && !ld_assume_kernel_is_set)
-  {
-    my_safe_printf_stderr("%s",
-      "You are running a statically-linked LinuxThreads binary on an NPTL\n"
-      "system. This can result in crashes on some distributions due to "
-      "LT/NPTL conflicts.\n"
-      "You should either build a dynamically-linked binary, "
-      "or force LinuxThreads\n"
-      "to be used with the LD_ASSUME_KERNEL environment variable.\n"
-      "Please consult the documentation for your distribution "
-      "on how to do that.\n");
-  }
-#endif
-
   if (locked_in_memory)
   {
     my_safe_printf_stderr("%s", "\n"
