@@ -1326,6 +1326,19 @@ public:
     values from the supplied list of Create_field objects.
   */
   void update_compressed_columns_info(const List<Create_field>& fields);
+private:
+  bool should_binlog_drop_if_temp_flag;
+
+public:
+  void set_binlog_drop_if_temp(bool should_binlog)
+  {
+    should_binlog_drop_if_temp_flag= should_binlog;
+  }
+
+  bool should_binlog_drop_if_temp(void) const
+  {
+    return should_binlog_drop_if_temp_flag;
+  }
 };
 
 
